/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import nl.cloudfarming.client.area.field.FieldAction;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.message.api.Message;
import nl.cloudfarming.client.message.api.MessageEvent;
import nl.cloudfarming.client.message.api.MessageEventKey;
import nl.cloudfarming.client.message.api.MessageEventProducer;
import nl.cloudfarming.client.model.Field;
import nl.cloudfarming.client.model.FieldPlan;
import nl.cloudfarming.client.model.ProductionUnit;
import nl.cloudfarming.client.planning.PlanEventKey;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import org.openide.DialogDescriptor;
import org.openide.DialogDisplayer;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.util.lookup.ServiceProvider;

/**
 * Link field to Production Unit action. Opens a Panel to Link a production unit
 * to a field. Will create a partfield in the process using
 * Field.createPartField(Field, ProductionUnit).
 *
 * @author Gerben Feenstra
 */
@ServiceProvider(service = FieldAction.class)
public class LinkProductionUnitToFieldController extends FieldAction {

    // STRING CONSTANTS 
    private static final String LINK_TO_PU_ACTION_NAME = NbBundle.getMessage(LinkProductionUnitToFieldController.class, "action.link_production_unit_to_field.name");
    private static final String DIALOG_DISPLAY_NAME = NbBundle.getMessage(LinkProductionUnitToFieldController.class, "action.link_production_unit_to_field.dialog.display_name");
    private static final String CONFIRM_MESSAGE_TITLE = NbBundle.getMessage(LinkProductionUnitToFieldController.class, "action.link_production_unit_to_field.confirm_message.title");
    private static final String CONFIRM_MESSAGE_FROM = NbBundle.getMessage(LinkProductionUnitToFieldController.class, "action.link_production_unit_to_field.confirm_message.from");
    private static final String BUTTON_OK = NbBundle.getMessage(LinkProductionUnitToFieldController.class, "action.link_production_unit_to_field.dialog.button.ok.text");
    private static final String BUTTON_CANCEL = NbBundle.getMessage(LinkProductionUnitToFieldController.class, "action.link_production_unit_to_field.dialog.button.cancel.text");    
    // Dialogs components
    private TacticalPlanningService service;
    private static final AppLogger LOG = AppLogFactory.getLogger(PlanProductionUnitController.class);
    private final MessageEventProducer messageEventProducer = new MessageEventProducer();
    private final EventProducer eventProducer = new EventProducer();

    public LinkProductionUnitToFieldController() {
        super(LINK_TO_PU_ACTION_NAME);
    }

    @Override
    public void actionPerformed(ActionEvent e) {

        // action can only be performed on a field, so field should be present in the actionsGlobalContext
        final Field field = Utilities.actionsGlobalContext().lookup(Field.class);
        assert field != null;
        service = getService();
        final LinkProductionUnitToFieldPanel panel = new LinkProductionUnitToFieldPanel(field);
        // check for fieldplans
        // no fieldplans --> warning, and stop building the panel
        int fieldPlans = service.getNumberOfFieldPlans();
        if (fieldPlans == 0) {
            //custom title, warning icon
            JOptionPane.showMessageDialog(panel,
                    NbBundle.getMessage(LinkProductionUnitToFieldController.class, "action.link_production_unit_to_field.dialog.noFieldplansFoundMessage"),
                    NbBundle.getMessage(LinkProductionUnitToFieldController.class, "action.link_production_unit_to_field.dialog.noFieldplansFoundTitle"),
                    JOptionPane.WARNING_MESSAGE);
        } else {
            //
            // create the ok and cancel buttons
            JButton ok = new JButton(BUTTON_OK);
            ok.setEnabled(false);
            JButton cancel = new JButton(BUTTON_CANCEL);
            //
            // populate the fieldplan box
            panel.populateFieldPlanBox(service.findAllFieldPlans());
            if (panel.getFieldplanBox().getItemCount() > 0) {
                panel.getFieldplanBox().addActionListener(new ActionListener() {

                    @Override
                    public void actionPerformed(ActionEvent e) {
                        panel.populateProductionUnitBox((FieldPlan) panel.getFieldplanBox().getSelectedItem());
                    }
                });
                panel.getFieldplanBox().setSelectedIndex(0);
                ok.setEnabled(true);
            }

            // Create the dialog and add the ok and cancel buttons
            DialogDescriptor dd = new DialogDescriptor(panel, DIALOG_DISPLAY_NAME);

            // OK and Cancel-Buttons
            dd.setOptions(new Object[]{ok, cancel});
            //
            // create the dialog and add actionlisteners for the ok and cancel button
            //
            final Dialog dialog = DialogDisplayer.getDefault().createDialog(dd);
            cancel.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent ae) {
                    dialog.setVisible(false);
                }
            });
            
            ok.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent ae) {
                    dialog.setVisible(false);
                    ProductionUnit productionUnit = (ProductionUnit) panel.getProductionUnitBox().getSelectedItem();
                    applyLink(field, productionUnit);
                }
            });            
            dialog.setVisible(true);
        }
    }

    private TacticalPlanningService getService() {
        return service == null ? service = Lookup.getDefault().lookup(TacticalPlanningService.class) : service;
    }

    public void setService(TacticalPlanningService service) {
        this.service = service;
    }



    protected final void applyLink(Field field, ProductionUnit selectedProductionUnit) {

        assert field != null;

        // Link PU to Field
        getService().createPartField(field, selectedProductionUnit);

        eventProducer.triggerEvent(PlanEventKey.LINK_FIELD_TO_PRODUCTION_UNIT);
        
        String confirm_message_text = NbBundle.getMessage(LinkProductionUnitToFieldController.class, "action.link_production_unit_to_field.confirm_message.message_text", selectedProductionUnit, field.getId(), field.getName());
        Message message = new Message();
        message.setTitle(CONFIRM_MESSAGE_TITLE);
        message.setMessageText(confirm_message_text);
        message.setDateReceived(new Date());
        message.setOrigin(CONFIRM_MESSAGE_FROM);
        messageEventProducer.triggerEvent(new MessageEvent(message, MessageEventKey.MESSAGE_NEW));
        LOG.info(confirm_message_text);
    }
    
    private class EventProducer extends BaseGuiEventProducer {

        public EventProducer() {
            super(PlanEventKey.class);
        }
    }
}
