/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical.wizard.spray;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.regex.Pattern;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.CropProtectionTask;
import nl.cloudfarming.client.model.ProductionUnit;
import nl.cloudfarming.client.planning.tactical.TacticalPlanningModule;
import org.netbeans.validation.api.builtin.Validators;
import org.netbeans.validation.api.ui.ValidationListener;
import org.netbeans.validation.api.ui.ValidationPanel;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

/**
 * Panel which displays the CropProtectionTasks
 * @author Merijn Zengers
 */
public class OperationPanel extends JPanel implements KeyListener {

    private static final String TITLE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.planning_visual_panel_crop_protection_edit_properties.title");
    private static final String ACREAGE_LABEL_TEXT = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.planning_visual_panel_crop_protection_edit_properties.label.acreage");
    
    private static final String LABEL_COLUMN_ID = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_id");
    private static final String LABEL_COLUMN_PESTICIDE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_pesticide");
    private static final String LABEL_COLUMN_APPLICATION = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_application");  
    private static final String LABEL_COLUMN_QUANTITY = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.planning_visual_panel_crop_protection_edit_properties.columnheader.quantity");
    private static final String LABEL_COLUMN_CONCENTRATION = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.planning_visual_panel_crop_protection_edit_properties.columnheader.concentration");

    private static final String TEXTFIELD_QUANTITY_NAME = "quantity";
    private static final String TEXTFIELD_CONCENTRATION_NAME = "concentration";

    private static final int LABEL_AUTH_ID_WIDTH = 75;
    private static final int LABEL_DOSAGE_UNIT_WIDTH = 40;
    private static final int LABEL_PESTICIDE_WIDTH = 150;
    private static final int TEXTFIELD_QUANTITY_WIDTH = 50;
    private static final int TEXTFIELD_CONCENTRATION_WIDTH = 60;
    private static final int LABEL_CONCENTRATION_DOSAGE_UNIT_WIDTH = 50;
    private static final int LABEL_ACREAGE_WIDTH = 150;

    private static final AppLogger LOG = AppLogFactory.getLogger(OperationPanel.class);
    
    private JPanel operationPanel;
    private JLabel acreageInfo;
    
    private JLabel pesticideColumnLabel;
    private JLabel quantityColumnLabel;
    private JLabel concentrationColumnLabel;
    private JLabel applicationColumnLabel;
    
    private final List<CropProtectionTask> cropProtectionTasks = new ArrayList<CropProtectionTask>();
    private ValidationPanel validationPanel;
    private JLabel authIdColumnValue;
    
    private JLabel authIdValue;
    private JLabel pesticideValue;
    private JLabel applicationValue;
    private InputField quantityValue;
    private JLabel quantityUnitLabel;
    private InputField concentrationValue;
    private JLabel concentrationDosageUnit;


    /** Creates new form OperationPanel */
    public OperationPanel() {
        initComponents();
    }
    
    /** 
     * This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        setLayout(new MigLayout("wrap 1"));
        
        ProductionUnit productionUnit = Utilities.actionsGlobalContext().lookup(ProductionUnit.class);
        assert productionUnit != null;

        // Intro text
        Double value = productionUnit.getAcreage();
        DecimalFormat format = new DecimalFormat("##.##");
        
        //int roundedValue = (int) (value * 100);
        //acreageValue = new JLabel(new Integer(roundedValue/100).toString());
        acreageInfo = new JLabel(ACREAGE_LABEL_TEXT + ": " + format.format(value));
        acreageInfo.setPreferredSize(new Dimension(LABEL_ACREAGE_WIDTH, 0));
        
        // Panel
        operationPanel = new JPanel();
        operationPanel.setLayout(new MigLayout("wrap 7")); // 7 components per CP row

        // Header Label declarations      
        authIdColumnValue = new JLabel(LABEL_COLUMN_ID); // 1 - ID
        pesticideColumnLabel = new JLabel(LABEL_COLUMN_PESTICIDE); // 2 - Pesticide (name)
        applicationColumnLabel = new JLabel(LABEL_COLUMN_APPLICATION); // 3 - Application
        quantityColumnLabel = new JLabel(LABEL_COLUMN_QUANTITY); // 4 - Quantity
        // 5 = Empty header
        concentrationColumnLabel = new JLabel(LABEL_COLUMN_CONCENTRATION); // 6 - Concentration
        // 7 = Empty header
        
        // Retrieve font from label (to be sure to get the default font)
        Font font = authIdColumnValue.getFont();
        Font boldFont = new Font(font.getName(), Font.BOLD, font.getSize());
        
        // Set font on all labels
        authIdColumnValue.setFont(boldFont);
        pesticideColumnLabel.setFont(boldFont);
        applicationColumnLabel.setFont(boldFont);
        quantityColumnLabel.setFont(boldFont);
        concentrationColumnLabel.setFont(boldFont);       
        
        //Validation
        validationPanel = new ValidationPanel();
        validationPanel.setInnerComponent(operationPanel);
        this.add(validationPanel);
        updatePanel();
    }

    /**
     * updates the panel according to the available cropProtectionTasks
     */
    private void updatePanel() {
        operationPanel.removeAll();
        
        operationPanel.add(acreageInfo, "span, wrap, gapbottom 10");
        operationPanel.add(authIdColumnValue);
        operationPanel.add(pesticideColumnLabel);
        operationPanel.add(applicationColumnLabel);
        operationPanel.add(quantityColumnLabel, "span 2");
        operationPanel.add(concentrationColumnLabel, "span 2, wrap");

        for (CropProtectionTask cropProtectionTask : cropProtectionTasks) {
            
            // 1 - Auth ID
            authIdValue = new JLabel(Integer.toString(cropProtectionTask.getCropProtection().getAuthorizationNumber()));
            
            // 2 - Pesticide name
            pesticideValue = new JLabel(cropProtectionTask.getCropProtection().getAgent().getName());
            
            // 3 - Application
            applicationValue = new JLabel(cropProtectionTask.getCropProtection().getApplicationMethod());
            
            // 4 - Quantity
            quantityValue = new InputField(cropProtectionTask);
            String quantity = cropProtectionTask.getQuantity() != null ? cropProtectionTask.getQuantity().toString() : "0.0";
            quantityValue.setText(quantity);
            quantityValue.setName(TEXTFIELD_QUANTITY_NAME);
            quantityValue.addKeyListener(this);

            // 5 - Quantity unit 
            quantityUnitLabel = new JLabel(cropProtectionTask.getCropProtection().getDosageUnit().getName());
            
            // 6 - Concentration
            concentrationValue = new InputField(cropProtectionTask);
            concentrationValue.setName(TEXTFIELD_CONCENTRATION_NAME);
            concentrationValue.addKeyListener(this);
            String concentration = cropProtectionTask.getConcentration() != null ? cropProtectionTask.getConcentration().toString() : "0.0";
            concentrationValue.setText(concentration);

            // 7 - Concentration Dosage
            concentrationDosageUnit = new JLabel(cropProtectionTask.getCropProtection().getConcentrationDosageUnit().getName());

            
            // Set sizes
            authIdValue.setPreferredSize(new Dimension(LABEL_AUTH_ID_WIDTH, 0));
            pesticideValue.setPreferredSize(new Dimension(LABEL_PESTICIDE_WIDTH, 0));     
            applicationValue.setPreferredSize(new Dimension(LABEL_PESTICIDE_WIDTH, 0));
            quantityValue.setPreferredSize(new Dimension(TEXTFIELD_QUANTITY_WIDTH, 0));
            quantityUnitLabel.setPreferredSize(new Dimension(LABEL_DOSAGE_UNIT_WIDTH, 0));
            concentrationValue.setPreferredSize(new Dimension(TEXTFIELD_CONCENTRATION_WIDTH, 0));
            concentrationDosageUnit.setPreferredSize(new Dimension(LABEL_CONCENTRATION_DOSAGE_UNIT_WIDTH, 0));

            operationPanel.add(authIdValue);
            operationPanel.add(pesticideValue);
            operationPanel.add(applicationValue);            
            operationPanel.add(quantityValue);
            operationPanel.add(quantityUnitLabel);
            operationPanel.add(concentrationValue);
            operationPanel.add(concentrationDosageUnit);

            cropProtectionTask.addPropertyChangeListener(quantityValue);
            cropProtectionTask.addPropertyChangeListener(concentrationValue);

            validationPanel.getValidationGroup().add(this.quantityValue, Validators.validNumber(Locale.getDefault()));         
            validationPanel.getValidationGroup().add(this.concentrationValue, Validators.validNumber(Locale.getDefault()));
            
        }
    }

    @Override
    public String getName() {
        return TITLE;
    }

    /*
     * Add a CropProtectionTask to the panel
     */
    public void addCropProtectionTask(CropProtectionTask cropProtectionTask) {
        cropProtectionTasks.add(cropProtectionTask);
        updatePanel();
    }

    /**
     * remove a cropProtectiontask from the panel
     * @param cropProtectionTask to be removed
     */
    public void removeCropProtectionTask(CropProtectionTask cropProtectionTask) {
        cropProtectionTasks.remove(cropProtectionTask);
        updatePanel();
    }

    /**
     * get all the availabel CropProtectionTasks from the panel
     * @return List <CropProtectiontask> all the displayed cropProtectionTasks
     */
    public List<CropProtectionTask> getCropProtectionTasks() {
        return cropProtectionTasks;
    }

    /**
     * invokes the validateAll method on the ValidationGroup
     */
    public void validateAll() {
        validationPanel.getValidationGroup().validateAll();
    }

    /**
     * Adds a validationListener to this panels ValidationGroup
     * @param validationListener 
     */
    public void addValidationListener(ValidationListener validationListener) {
        validationPanel.getValidationGroup().add(validationListener);
    }

    /**
     * not used
     * @param e 
     */
    @Override
    public void keyTyped(KeyEvent e) {
    }

    /*
     * Not used
     */
    @Override
    public void keyPressed(KeyEvent e) {
    }

    /**
     * Fired if a key is relaeased on one of the two input fields
     * @param e 
     */
    @Override
    public void keyReleased(KeyEvent e) {
        validateAll();
        if (Pattern.matches("\\d*", Character.toString(e.getKeyChar()))) {
            if (e.getSource() instanceof InputField) {
                InputField inputField = (InputField) e.getSource();
                try {
                    Double value = Double.parseDouble(inputField.getText());
                    if (inputField.getName().equals(TEXTFIELD_QUANTITY_NAME)) {
                        inputField.getCropProtectionTask().setQuantity(value);
                    } else if (inputField.getName().equals(TEXTFIELD_CONCENTRATION_NAME)) {
                        inputField.getCropProtectionTask().setConcentration(value);
                    }
                } catch (NumberFormatException ex) {
                    LOG.error("Could not Convert string to number", ex);
                }
            }
        }
    }

    /**
     * implementation of JTextField. Holds the crpProtectionTask for which this inputField is displayed
     */
    private class InputField extends JTextField implements PropertyChangeListener {

        private CropProtectionTask cropProtectionTask;

        public InputField(CropProtectionTask cropProtectionTask) {
            this.cropProtectionTask = cropProtectionTask;
        }

        public CropProtectionTask getCropProtectionTask() {
            return cropProtectionTask;
        }

        @Override
        public void setText(String text) {
            DecimalFormat format = new DecimalFormat("#.###");
            try {
                super.setText(format.format(Double.parseDouble(text)));
            } catch (NumberFormatException ex) {
                LOG.error("Could not Convert string to number", ex);
            }
        }

        @Override
        public void propertyChange(PropertyChangeEvent evt) {
            if (evt.getPropertyName().equals(getName())) {
                this.setText(evt.getNewValue().toString());
            }
        }
    }
}
