/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical.wizard.spray;

import com.vividsolutions.jts.io.ParseException;
import java.awt.Component;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Date;
import java.util.Map;
import javax.swing.JComponent;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.CropProtectionTask;
import nl.cloudfarming.client.model.Operation;
import nl.cloudfarming.client.model.PartField;
import nl.cloudfarming.client.model.ProductionUnit;
import nl.cloudfarming.client.planning.PlanEventKey;
import nl.cloudfarming.client.planning.tactical.TacticalPlanningModule;
import nl.cloudfarming.client.planning.tactical.TacticalPlanningService;
import nl.cloudfarming.client.planning.tactical.TacticalPlanningServiceImpl;
import nl.cloudfarming.client.planning.tactical.wizard.PlandateController;
import nl.cloudfarming.client.planning.wizard.PlanningWizardAction;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import nl.cloudfarming.eventbus.GuiEvent;
import nl.cloudfarming.eventbus.GuiEventKey;
import org.openide.WizardDescriptor;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Merijn Zengers
 */
@ServiceProvider(service=PlanningWizardAction.class)
public class SprayWizardAction implements nl.cloudfarming.client.planning.wizard.PlanningWizardAction{
    private static final int PREFFERED_WIDTH = 600;
    private static final int PREFFERED_HEIGHT = 400;
    
    private TacticalPlanningService service;
    private WizardDescriptor.Panel[] panelControllers;
    private static final String DISPLAY_NAME = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.planning_wizard_action.name");
    private static final String OPERATION_NAME = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.planning_wizard_action.operation_name");
    private final PlanningEventProducer planningEventProducer = new PlanningEventProducer();
    private static final AppLogger LOG = AppLogFactory.getLogger(TacticalPlanningModule.class);
    
    @Override
    public String getWizardName() {
        return DISPLAY_NAME;
    }

    /**
     * Initialize panels representing individual wizard's steps and sets
     * various properties for them influencing wizard appearance.
     */
    @Override
    public WizardDescriptor.Panel[] getPanels() {
        if (panelControllers == null) {
            panelControllers = new WizardDescriptor.Panel[]{
                new PlandateController(),
                new CropProtectionController(),
                new OperationController(),
                new SummaryController()
            };
            
            String[] steps = new String[panelControllers.length];
            for (int i = 0; i < panelControllers.length; i++) {
                Component c = panelControllers[i].getComponent();
                c.setPreferredSize(new Dimension(PREFFERED_WIDTH, PREFFERED_HEIGHT));
                // Default step name to component name of panel. Mainly useful
                // for getting the name of the target chooser to appear in the
                // list of steps.
                steps[i] = c.getName();
                if (c instanceof JComponent) { // assume Swing components
                    JComponent jc = (JComponent) c;
                    // Sets step number of a component
                    
                    jc.putClientProperty("WizardPanel_contentSelectedIndex", Integer.valueOf(i));
                    // Sets steps names for a panel
                    jc.putClientProperty("WizardPanel_contentData", steps);
                    // Turn on subtitle creation on each step
                    jc.putClientProperty("WizardPanel_autoWizardStyle", Boolean.TRUE);
                    // Show steps on the left side with the image on the background
                    jc.putClientProperty("WizardPanel_contentDisplayed", Boolean.TRUE);
                    // Turn on numbering of all steps
                    jc.putClientProperty("WizardPanel_contentNumbered", Boolean.TRUE);
                }
            }
        }
        return panelControllers;
    }

    @Override
    public void save(WizardDescriptor wizardDescriptor) {
        Operation operation;
            try {
                operation = saveWizardContent(wizardDescriptor);
                planningEventProducer.triggerEvent(new OperationPlanEvent(operation, PlanEventKey.NEW_OPERATION));
            } catch (CropProtectionNotFoundException ex) {
                LOG.error(ex.getMessage(), ex);
            } catch (ProductionUnitNotFoundException ex) {
                LOG.error(ex.getMessage(), ex);
            } catch (ParseException ex) {
                LOG.error(ex.getMessage(), ex);
            }
    }
    
    /**
     * Method which saves the data in the wizard
     * @param wizardDescriptor
     * @return 
     */
    protected Operation saveWizardContent(WizardDescriptor wizardDescriptor) throws ParseException, ProductionUnitNotFoundException, CropProtectionNotFoundException {
        ArrayList<CropProtectionTask> selectedCropProtections =  (ArrayList<CropProtectionTask>) wizardDescriptor.getProperty(OperationController.CROP_PROTECTION_TASK_PROPERTIES);
        Date planDate = (Date) wizardDescriptor.getProperty(PlandateController.PLANDATE);

        ProductionUnit productionUnit = Utilities.actionsGlobalContext().lookup(ProductionUnit.class);

        if (productionUnit != null) {
            Operation operation = null;
            operation = getService().createOperation(selectedCropProtections.get(0).getOperation());
            for (PartField partField : productionUnit.getPartFields()) {
                if (selectedCropProtections != null) {
                    for(CropProtectionTask cropProtectionTask : selectedCropProtections) {
                        cropProtectionTask.setPlanDate(planDate);
                        cropProtectionTask.setPartField(partField);
                        operation.setName(OPERATION_NAME);
                        cropProtectionTask.setOperation(operation);
                        getService().createCropProtectionTask(cropProtectionTask);
                    }
                } else{
                    throw new CropProtectionNotFoundException("CropProtection could not be found when trying to save wizard data");
                }
            }

            return operation;
        } else {
            throw new ProductionUnitNotFoundException("ProductionUnit not found while trying to save wizard data");
        }

    }
    /**
     * Return service for tactical planning
     * @return The service for tactical planning
     */
    protected TacticalPlanningService getService() {
        if (service == null) {
            service = Lookup.getDefault().lookup(TacticalPlanningService.class);
        }
        return service;
    }
    
    @Override
    public String toString(){
        return OPERATION_NAME;
    }

    /**
     * Set service for tactical planning
     * @param service Service to set
     */
    public void setService(TacticalPlanningServiceImpl service) {
        this.service = service;
    }

    @Override
    public void clear(WizardDescriptor wizardDescriptor) {
        // clear settings of panels
        Map wdMap = wizardDescriptor.getProperties();
        for(Object object:wdMap.keySet()) {
            wizardDescriptor.putProperty(object.toString(),null);
            wizardDescriptor.getProperty(object.toString());
        }
        panelControllers = null;
        
    }

    private class OperationPlanEvent implements GuiEvent<Operation> {

        private final Operation operation;
        private final PlanEventKey key;

        /**
         * Constructor
         * @param fieldPlan: the fieldplan for which the event is triggered
         * @param key: the event key. Which action has been done to trigger this event. 
         */
        public OperationPlanEvent(Operation operation, PlanEventKey key) {
            this.operation = operation;
            this.key = key;
        }

        @Override
        public GuiEventKey getKey() {
            return key;
        }

        @Override
        public Operation getContent() {
            return operation;
        }
    }

    private class PlanningEventProducer extends BaseGuiEventProducer {

        public PlanningEventProducer() {
            super(PlanEventKey.class);
        }
    }
    
}
